package bezier;

import edu.princeton.cs.StdDraw;
import java.awt.geom.Point2D;

public class Bezier {

    public static Point2D randompoint() {
        double x = 2 * Math.random() - 1;
        double y = 2 * Math.random() - 1;

        return new Point2D.Double(x, y);

    } // randompoint()

    public static double avg(double a, double b, double t) {
        return (1 - t) * a + (t * b);
    } // avg( double, double, double )

    public static Point2D avg(Point2D a, Point2D b, double t) {
        double x = avg(a.getX(), b.getX(), t);
        double y = avg(a.getY(), b.getY(), t);
        return new Point2D.Double(x, y);
    } // avg( Point2D, Point2D, double )
    
    public static Point2D avg( Point2D a, Point2D b, Point2D c, double t ) {
        return avg( avg( a, b, t), avg(b, c, t), t);
    } // avg( Point2D, Point2D, Point2D, double )
    
    public static Point2D avg( Point2D a, Point2D b, Point2D c, Point2D d, double t ) {
        return avg( avg( a, b, c, t), avg( b, c, d, t), t);
    } // avg( Point2D, Point2D, Point2D, Point2D, double )

    public static void main(String[] args) {

        StdDraw.setCanvasSize(512, 512);
        StdDraw.setScale(-1, 1);
        Point2D[] points = new Point2D[4];

        for (int i = 0; i < 4; i++) {
            points[i] = randompoint();
        } // for

        for (Point2D p : points) {
            StdDraw.filledCircle(p.getX(), p.getY(), .02);
        } // for

//        Point2D previousPoint = points[0];
//        for (int i = 1; i < 4; i++) {
//            Point2D currentPoint = points[i];
//            StdDraw.line(previousPoint.getX(), previousPoint.getY(),
//                    currentPoint.getX(), currentPoint.getY());
//            previousPoint = currentPoint;
//        } // for
        Point2D pp = points[0];
        for (int i = 0; i < 128; i++) {
            double t = ((double) i) / 128;
            Point2D cp = avg(points[0], points[1], points[2], points[3], t);

            if (i % 2 == 0) {
                StdDraw.line(pp.getX(), pp.getY(), cp.getX(), cp.getY());
            } // if
            
            pp = cp;
        } // for

    }

} // Bezier
